<?php

namespace Litepie\Setting\Actions;

use Illuminate\Support\Carbon;
use Illuminate\Support\Str;
use Litepie\Actions\Concerns\AsAction;
use Litepie\Actions\Traits\LogsActions;
use Litepie\Notification\Traits\SendNotification;
use Litepie\Setting\Models\Setting;


class SettingAction
{
    use AsAction;

    protected $model;
    protected $namespace = 'litepie.setting.setting';
    protected $eventClass = \Litepie\Setting\Events\SettingAction::class;
    protected $action;
    protected $function;
    protected $request;

    public function handle(string $action, Setting $setting, array $request = [])
    {
        $this->action = $action;
        $this->request = $request;
        $this->model = $setting;
        $this->function = Str::camel($action);
        $this->executeAction();
        return $this->model;

    }


    public function store(Setting $setting, array $request)
    {
        $attributes = $request;
        $attributes['user_id'] = user_id();
        $attributes['user_type'] = user_type();
        $setting = $setting->create($attributes);
        return $setting;
    }

    public function update(Setting $setting, array $request)
    {
        $attributes = $request;
        $setting->update($attributes);
        return $setting;
    }

    public function destroy(Setting $setting, array $request)
    {
        $setting->delete();
        return $setting;
    }

    public function copy(Setting $setting, array $request)
    {
        $count = $request['count'] ?: 1;

        if ($count == 1) {
            $setting = $setting->replicate();
            $setting->created_at = Carbon::now();
            $setting->save();
            return $setting;
        }

        for ($i = 1; $i <= $count; $i++) {
            $new = $setting->replicate();
            $new->created_at = Carbon::now();
            $new->save();
        }

        return $setting;
    }


}
