<?php

namespace Litepie\Setting\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Litepie\Setting\Events\SettingAction as SettingActionEvent;
use Litepie\Setting\Notifications\SettingAction as SettingActionNotification;
use Litepie\Actions\Concerns\AsAction;

class SettingAction
{
    use AsAction;

    private $allowedActions = [
        'before' => [],
        'after' => ['create'],
    ];

    /**
     * Handle the SettingActionEvent.
     *
     * @param   SettingActionEvent  $event
     * @return mixed
     */
    public function handle(SettingActionEvent $event)
    {
        $function = Str::camel($event->action);
        return $this->$function($event);
    }

    /**
     * Create a new $setting.
     *
     * @param   SettingActionEvent  $event
     * @return void
     */
    public function create(SettingActionEvent $event)
    {
        $client = $event->$setting->client;
        Notification::send($client, new SettingActionNotification($event));
    }

    /**
     * Handle the SettingActionEvent as a listener.
     *
     * @param   SettingActionEvent  $event
     * @return mixed
     */
    public function asListener(SettingActionEvent $event)
    {
        if ($this->isAllowed($event)) {
            return $this->handle($event);
        }
    }

    /**
     * Check if the event action is allowed.
     *
     * @param   SettingActionEvent  $event
     * @return bool
     */
    private function isAllowed(SettingActionEvent $event)
    {
        if ($event->when == 'before' &&
            !in_array($event->action, $this->allowedActions['before'])) {
            return false;
        }

        if (($event->when == 'after' &&
            !in_array($event->action, $this->allowedActions['after']))
        ) {
            return false;
        }

        return true;
    }
}
