<?php

namespace Litepie\Setting\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Litepie\Setting\Events\SettingWorkflow as SettingWorkflowEvent;
use Litepie\Setting\Notifications\SettingWorkflow as SettingWorkflowNotification;
use Litepie\Actions\Concerns\AsAction;

class SettingWorkflow
{
    use AsAction;

    private $allowedTransitions = [
        'before' => [],
        'after' => ['publish', 'submit', 'approve'],
    ];

    public function handle(SettingWorkflowEvent $event)
    {
        $function = Str::camel($event->transition);
        return $this->$function($event);
    }

    /**
     * Sends a notification to the client when the $setting is submitted.
     *
     * @param SettingWorkflowEvent $event The event object.
     * @return void
     */
    public function submit(SettingWorkflowEvent $event)
    {
        $client = $event->$setting->client;
        Notification::send($client, new SettingWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $setting is published.
     *
     * @param SettingWorkflowEvent $event The event object.
     * @return void
     */
    public function publish(SettingWorkflowEvent $event)
    {
        $client = $event->$setting->client;
        Notification::send($client, new SettingWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $setting is approved.
     *
     * @param SettingWorkflowEvent $event The event object.
     * @return void
     */
    public function approve(SettingWorkflowEvent $event)
    {
        $client = $event->$setting->client;
        Notification::send($client, new SettingWorkflowNotification($event));
    }

    /**
     * Handles the $setting workflow event as a listener.
     *
     * @param SettingWorkflowEvent $event The event object.
     * @return void
     */
    public function asListener(SettingWorkflowEvent $event)
    {
        if (($event->when == 'before' &&
            in_array($event->transition, $this->allowedTransitions['before']) ||
            $event->when == 'after' &&
            in_array($event->transition, $this->allowedTransitions['after']))
        ) {
            return $this->handle($event);
        }
    }
}
